@@ -26,31 +26,34 @@ from pm4py.objects.petri.importer import pnml as petri_importer
 from pm4py.objects.log import log as log_implementation
 from pm4py.util.xes_constants import DEFAULT_NAME_KEY
 from pm4py.objects.petri.synchronous_product import construct_cost_aware
-from pm4py.objects.petri.utils import construct_trace_net_cost_aware, decorate_places_preset_trans, decorate_transitions_prepostset
-from pm4py.util.constants import PARAMETER_CONSTANT_ACTIVITY_KEY
-from pm4py.util.lp import factory as lp_solver_factory
+from pm4py.objects.petri.utils import construct_trace_net_cost_aware, decorate_places_preset_trans, \
+    decorate_transitions_prepostset
+from pm4py.util.lp import solver as lp_solver
 from pm4py.objects.petri import align_utils as utils
+from pm4py.util import exec_utils
+from enum import Enum
+import sys
+from pm4py.util.constants import PARAMETER_CONSTANT_ACTIVITY_KEY
 
-PARAM_TRACE_COST_FUNCTION = 'trace_cost_function'
-PARAM_MODEL_COST_FUNCTION = 'model_cost_function'
-PARAM_SYNC_COST_FUNCTION = 'sync_cost_function'
 
-PARAM_ALIGNMENT_RESULT_IS_SYNC_PROD_AWARE = 'ret_tuple_as_trans_desc'
-PARAM_TRACE_NET_COSTS = "trace_net_costs"
+class Parameters(Enum):
+    PARAM_TRACE_COST_FUNCTION = 'trace_cost_function'
+    PARAM_MODEL_COST_FUNCTION = 'model_cost_function'
+    PARAM_SYNC_COST_FUNCTION = 'sync_cost_function'
+    PARAM_ALIGNMENT_RESULT_IS_SYNC_PROD_AWARE = 'ret_tuple_as_trans_desc'
+    PARAM_TRACE_NET_COSTS = "trace_net_costs"
+    TRACE_NET_CONSTR_FUNCTION = "trace_net_constr_function"
+    TRACE_NET_COST_AWARE_CONSTR_FUNCTION = "trace_net_cost_aware_constr_function"
+    PARAM_MAX_ALIGN_TIME_TRACE = "max_align_time_trace"
+    PARAM_MAX_ALIGN_TIME = "max_align_time"
+    PARAMETER_VARIANT_DELIMITER = "variant_delimiter"
+    ACTIVITY_KEY = PARAMETER_CONSTANT_ACTIVITY_KEY
+    VARIANTS_IDX = "variants_idx"
 
-TRACE_NET_CONSTR_FUNCTION = "trace_net_constr_function"
-TRACE_NET_COST_AWARE_CONSTR_FUNCTION = "trace_net_cost_aware_constr_function"
 
-PARAM_MAX_ALIGN_TIME_TRACE = "max_align_time_trace"
-DEFAULT_MAX_ALIGN_TIME_TRACE = sys.maxsize
-PARAM_MAX_ALIGN_TIME = "max_align_time"
-DEFAULT_MAX_ALIGN_TIME = sys.maxsize
-
-PARAMETER_VARIANT_DELIMITER = "variant_delimiter"
-DEFAULT_VARIANT_DELIMITER = ","
-
-PARAMETERS = [PARAM_TRACE_COST_FUNCTION, PARAM_MODEL_COST_FUNCTION, PARAM_SYNC_COST_FUNCTION,
-              pm4pyutil.constants.PARAMETER_CONSTANT_ACTIVITY_KEY]
+PARAM_TRACE_COST_FUNCTION = Parameters.PARAM_TRACE_COST_FUNCTION.value
+PARAM_MODEL_COST_FUNCTION = Parameters.PARAM_MODEL_COST_FUNCTION.value
+PARAM_SYNC_COST_FUNCTION = Parameters.PARAM_SYNC_COST_FUNCTION.value
 
 
 def get_best_worst_cost(petri_net, initial_marking, final_marking, parameters=None):
@@ -75,8 +78,10 @@ def get_best_worst_cost(petri_net, initial_marking, final_marking, parameters=No
         parameters = {}
     trace = log_implementation.Trace()
     new_parameters = copy(parameters)
-    if PARAM_TRACE_COST_FUNCTION not in new_parameters or len(new_parameters[PARAM_TRACE_COST_FUNCTION]) < len(trace):
-        new_parameters[PARAM_TRACE_COST_FUNCTION] = list(
+
+    trace_cost_function = exec_utils.get_param_value(Parameters.PARAM_TRACE_COST_FUNCTION, new_parameters, [])
+    if len(trace_cost_function) < len(trace):
+        new_parameters[Parameters.PARAM_TRACE_COST_FUNCTION] = list(
             map(lambda e: utils.STD_MODEL_LOG_MOVE_COST, trace))
 
     best_worst = pm4py.algo.conformance.alignments.versions.state_equation_a_star.apply(trace,
@@ -101,12 +106,12 @@ def apply(trace, petri_net, initial_marking, final_marking, parameters=None):
     initial_marking: :class:`pm4py.objects.petri.net.Marking` initial marking in the Petri net
     final_marking: :class:`pm4py.objects.petri.net.Marking` final marking in the Petri net
     parameters: :class:`dict` (optional) dictionary containing one of the following:
-        PARAM_TRACE_COST_FUNCTION: :class:`list` (parameter) mapping of each index of the trace to a positive cost value
-        PARAM_MODEL_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
+        Parameters.PARAM_TRACE_COST_FUNCTION: :class:`list` (parameter) mapping of each index of the trace to a positive cost value
+        Parameters.PARAM_MODEL_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
         model cost
-        PARAM_SYNC_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
+        Parameters.PARAM_SYNC_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
         synchronous costs
-        PARAM_ACTIVITY_KEY: :class:`str` (parameter) key to use to identify the activity described by the events
+        Parameters.ACTIVITY_KEY: :class:`str` (parameter) key to use to identify the activity described by the events
 
     Returns
     -------
@@ -115,21 +120,22 @@ def apply(trace, petri_net, initial_marking, final_marking, parameters=None):
     if parameters is None:
         parameters = {}
 
-    activity_key = DEFAULT_NAME_KEY if parameters is None or PARAMETER_CONSTANT_ACTIVITY_KEY not in parameters else \
-        parameters[
-            pm4pyutil.constants.PARAMETER_CONSTANT_ACTIVITY_KEY]
-    if parameters is None or PARAM_TRACE_COST_FUNCTION not in parameters or PARAM_MODEL_COST_FUNCTION not in parameters or PARAM_SYNC_COST_FUNCTION not in parameters:
-        trace_net_constr_function = parameters[
-            TRACE_NET_CONSTR_FUNCTION] if TRACE_NET_CONSTR_FUNCTION in parameters else petri.utils.construct_trace_net
-        trace_net, trace_im, trace_fm = trace_net_constr_function(trace, activity_key=activity_key)
+    activity_key = exec_utils.get_param_value(Parameters.ACTIVITY_KEY, parameters, DEFAULT_NAME_KEY)
+    trace_cost_function = exec_utils.get_param_value(Parameters.PARAM_TRACE_COST_FUNCTION, parameters, None)
+    model_cost_function = exec_utils.get_param_value(Parameters.PARAM_MODEL_COST_FUNCTION, parameters, None)
+    sync_cost_function = exec_utils.get_param_value(Parameters.PARAM_SYNC_COST_FUNCTION, parameters, None)
+    trace_net_constr_function = exec_utils.get_param_value(Parameters.TRACE_NET_CONSTR_FUNCTION, parameters,
+                                                           petri.utils.construct_trace_net)
+    trace_net_cost_aware_constr_function = exec_utils.get_param_value(Parameters.TRACE_NET_COST_AWARE_CONSTR_FUNCTION,
+                                                                      parameters, construct_trace_net_cost_aware)
 
+    if trace_cost_function is None or model_cost_function is None or sync_cost_function is None:
+        trace_net, trace_im, trace_fm = trace_net_constr_function(trace, activity_key=activity_key)
     else:
-        trace_net_cost_aware_constr_function = parameters[
-            TRACE_NET_COST_AWARE_CONSTR_FUNCTION] if TRACE_NET_COST_AWARE_CONSTR_FUNCTION in parameters else construct_trace_net_cost_aware
-        trace_net, trace_im, trace_fm, parameters[PARAM_TRACE_NET_COSTS] = trace_net_cost_aware_constr_function(trace,
-                                                                                                                parameters[
-                                                                                                                    PARAM_TRACE_COST_FUNCTION],
-                                                                                                                activity_key=activity_key)
+        trace_net, trace_im, trace_fm, parameters[
+            Parameters.PARAM_TRACE_NET_COSTS] = trace_net_cost_aware_constr_function(trace,
+                                                                                     trace_cost_function,
+                                                                                     activity_key=activity_key)
 
     return apply_trace_net(petri_net, initial_marking, final_marking, trace_net, trace_im, trace_fm, parameters)
 
@@ -161,8 +167,8 @@ def apply_from_variant(variant, petri_net, initial_marking, final_marking, param
         parameters[
             pm4pyutil.constants.PARAMETER_CONSTANT_ACTIVITY_KEY]
     trace = log_implementation.Trace()
-    variant_delimiter = parameters[
-        PARAMETER_VARIANT_DELIMITER] if PARAMETER_VARIANT_DELIMITER in parameters else DEFAULT_VARIANT_DELIMITER
+    variant_delimiter = exec_utils.get_param_value(Parameters.PARAMETER_VARIANT_DELIMITER, parameters,
+                                                   ",")
     variant_split = variant.split(variant_delimiter) if type(variant) is str else variant
     for i in range(len(variant_split)):
         trace.append(log_implementation.Event({activity_key: variant_split[i]}))
@@ -225,14 +231,15 @@ def apply_from_variants_list(var_list, petri_net, initial_marking, final_marking
     if parameters is None:
         parameters = {}
     start_time = time.time()
-    max_align_time = parameters[PARAM_MAX_ALIGN_TIME] if PARAM_MAX_ALIGN_TIME in parameters else DEFAULT_MAX_ALIGN_TIME
-    max_align_time_case = parameters[
-        PARAM_MAX_ALIGN_TIME_TRACE] if PARAM_MAX_ALIGN_TIME_TRACE in parameters else DEFAULT_MAX_ALIGN_TIME_TRACE
+    max_align_time = exec_utils.get_param_value(Parameters.PARAM_MAX_ALIGN_TIME, parameters,
+                                                sys.maxsize)
+    max_align_time_trace = exec_utils.get_param_value(Parameters.PARAM_MAX_ALIGN_TIME_TRACE, parameters,
+                                                     sys.maxsize)
     dictio_alignments = {}
     for varitem in var_list:
-        this_max_align_time = min(max_align_time_case, (max_align_time - (time.time() - start_time)) * 0.5)
+        this_max_align_time = min(max_align_time_trace, (max_align_time - (time.time() - start_time)) * 0.5)
         variant = varitem[0]
-        parameters[PARAM_MAX_ALIGN_TIME_TRACE] = this_max_align_time
+        parameters[Parameters.PARAM_MAX_ALIGN_TIME_TRACE] = this_max_align_time
         dictio_alignments[variant] = apply_from_variant(variant, petri_net, initial_marking, final_marking,
                                                         parameters=parameters)
     return dictio_alignments
@@ -300,13 +307,13 @@ def apply_trace_net(petri_net, initial_marking, final_marking, trace_net, trace_
         initial_marking: :class:`pm4py.objects.petri.net.Marking` initial marking in the Petri net
         final_marking: :class:`pm4py.objects.petri.net.Marking` final marking in the Petri net
         parameters: :class:`dict` (optional) dictionary containing one of the following:
-            PARAM_TRACE_COST_FUNCTION: :class:`list` (parameter) mapping of each index of the trace to a positive cost value
-            PARAM_MODEL_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
+            Parameters.PARAM_TRACE_COST_FUNCTION: :class:`list` (parameter) mapping of each index of the trace to a positive cost value
+            Parameters.PARAM_MODEL_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
             model cost
-            PARAM_SYNC_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
+            Parameters.PARAM_SYNC_COST_FUNCTION: :class:`dict` (parameter) mapping of each transition in the model to corresponding
             synchronous costs
-            PARAM_ACTIVITY_KEY: :class:`str` (parameter) key to use to identify the activity described by the events
-            PARAM_TRACE_NET_COSTS: :class:`dict` (parameter) mapping between transitions and costs
+            Parameters.ACTIVITY_KEY: :class:`str` (parameter) key to use to identify the activity described by the events
+            Parameters.PARAM_TRACE_NET_COSTS: :class:`dict` (parameter) mapping between transitions and costs
 
         Returns
         -------
@@ -315,10 +322,15 @@ def apply_trace_net(petri_net, initial_marking, final_marking, trace_net, trace_
     if parameters is None:
         parameters = {}
 
-    ret_tuple_as_trans_desc = parameters[
-        PARAM_ALIGNMENT_RESULT_IS_SYNC_PROD_AWARE] if PARAM_ALIGNMENT_RESULT_IS_SYNC_PROD_AWARE in parameters else False
+    ret_tuple_as_trans_desc = exec_utils.get_param_value(Parameters.PARAM_ALIGNMENT_RESULT_IS_SYNC_PROD_AWARE,
+                                                         parameters, False)
 
-    if parameters is None or PARAM_TRACE_COST_FUNCTION not in parameters or PARAM_MODEL_COST_FUNCTION not in parameters or PARAM_SYNC_COST_FUNCTION not in parameters:
+    trace_cost_function = exec_utils.get_param_value(Parameters.PARAM_TRACE_COST_FUNCTION, parameters, None)
+    model_cost_function = exec_utils.get_param_value(Parameters.PARAM_MODEL_COST_FUNCTION, parameters, None)
+    sync_cost_function = exec_utils.get_param_value(Parameters.PARAM_SYNC_COST_FUNCTION, parameters, None)
+    trace_net_costs = exec_utils.get_param_value(Parameters.PARAM_TRACE_NET_COSTS, parameters, None)
+
+    if trace_cost_function is None or model_cost_function is None or sync_cost_function is None:
         sync_prod, sync_initial_marking, sync_final_marking = petri.synchronous_product.construct(trace_net, trace_im,
                                                                                                   trace_fm, petri_net,
                                                                                                   initial_marking,
@@ -330,14 +342,14 @@ def apply_trace_net(petri_net, initial_marking, final_marking, trace_net, trace_
         for t_trace in trace_net.transitions:
             for t_model in petri_net.transitions:
                 if t_trace.label == t_model.label:
-                    revised_sync[(t_trace, t_model)] = parameters[PARAM_SYNC_COST_FUNCTION][t_model]
+                    revised_sync[(t_trace, t_model)] = sync_cost_function[t_model]
 
         sync_prod, sync_initial_marking, sync_final_marking, cost_function = construct_cost_aware(
             trace_net, trace_im, trace_fm, petri_net, initial_marking, final_marking, utils.SKIP,
-            parameters[PARAM_TRACE_NET_COSTS], parameters[PARAM_MODEL_COST_FUNCTION], revised_sync)
+            trace_net_costs, model_cost_function, revised_sync)
 
-    max_align_time_trace = parameters[
-        PARAM_MAX_ALIGN_TIME_TRACE] if PARAM_MAX_ALIGN_TIME_TRACE in parameters else DEFAULT_MAX_ALIGN_TIME_TRACE
+    max_align_time_trace = exec_utils.get_param_value(Parameters.PARAM_MAX_ALIGN_TIME_TRACE, parameters,
+                                                     sys.maxsize)
 
     return apply_sync_prod(sync_prod, sync_initial_marking, sync_final_marking, cost_function,
                            utils.SKIP, ret_tuple_as_trans_desc=ret_tuple_as_trans_desc,
@@ -345,7 +357,7 @@ def apply_trace_net(petri_net, initial_marking, final_marking, trace_net, trace_
 
 
 def apply_sync_prod(sync_prod, initial_marking, final_marking, cost_function, skip, ret_tuple_as_trans_desc=False,
-                    max_align_time_trace=DEFAULT_MAX_ALIGN_TIME_TRACE):
+                    max_align_time_trace=sys.maxsize):
     """
     Performs the basic alignment search on top of the synchronous product net, given a cost function and skip-symbol
 
@@ -367,7 +379,7 @@ def apply_sync_prod(sync_prod, initial_marking, final_marking, cost_function, sk
 
 
 def __search(sync_net, ini, fin, cost_function, skip, ret_tuple_as_trans_desc=False,
-             max_align_time_trace=DEFAULT_MAX_ALIGN_TIME_TRACE):
+             max_align_time_trace=sys.maxsize):
     start_time = time.time()
 
     decorate_transitions_prepostset(sync_net)
@@ -384,7 +396,7 @@ def __search(sync_net, ini, fin, cost_function, skip, ret_tuple_as_trans_desc=Fa
     cost_vec = [x * 1.0 for x in cost_vec]
 
     use_cvxopt = False
-    if lp_solver_factory.DEFAULT_LP_SOLVER_VARIANT == lp_solver_factory.CVXOPT_SOLVER_CUSTOM_ALIGN or lp_solver_factory.DEFAULT_LP_SOLVER_VARIANT == lp_solver_factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP:
+    if lp_solver.DEFAULT_LP_SOLVER_VARIANT == lp_solver.CVXOPT_SOLVER_CUSTOM_ALIGN or lp_solver.DEFAULT_LP_SOLVER_VARIANT == lp_solver.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP:
         use_cvxopt = True
 
     if use_cvxopt:
@@ -396,8 +408,10 @@ def __search(sync_net, ini, fin, cost_function, skip, ret_tuple_as_trans_desc=Fa
         h_cvx = matrix(h_cvx)
         cost_vec = matrix(cost_vec)
 
-    h, x = utils.__compute_exact_heuristic_new_version(sync_net, a_matrix, h_cvx, g_matrix, cost_vec, incidence_matrix, ini,
-                                                 fin_vec, lp_solver_factory.DEFAULT_LP_SOLVER_VARIANT, use_cvxopt=use_cvxopt)
+    h, x = utils.__compute_exact_heuristic_new_version(sync_net, a_matrix, h_cvx, g_matrix, cost_vec, incidence_matrix,
+                                                       ini,
+                                                       fin_vec, lp_solver.DEFAULT_LP_SOLVER_VARIANT,
+                                                       use_cvxopt=use_cvxopt)
     ini_state = utils.SearchTuple(0 + h, 0, h, ini, None, None, x, True)
     open_set = [ini_state]
     heapq.heapify(open_set)
@@ -423,8 +437,9 @@ def __search(sync_net, ini, fin, cost_function, skip, ret_tuple_as_trans_desc=Fa
 
         while not curr.trust:
             h, x = utils.__compute_exact_heuristic_new_version(sync_net, a_matrix, h_cvx, g_matrix, cost_vec,
-                                                         incidence_matrix, curr.m,
-                                                         fin_vec, lp_solver_factory.DEFAULT_LP_SOLVER_VARIANT, use_cvxopt=use_cvxopt)
+                                                               incidence_matrix, curr.m,
+                                                               fin_vec, lp_solver.DEFAULT_LP_SOLVER_VARIANT,
+                                                               use_cvxopt=use_cvxopt)
 
             # 11/10/19: shall not a state for which we compute the exact heuristics be
             # by nature a trusted solution?
@@ -435,7 +450,7 @@ def __search(sync_net, ini, fin, cost_function, skip, ret_tuple_as_trans_desc=Fa
             current_marking = curr.m
 
         # max allowed heuristics value (27/10/2019, due to the numerical instability of some of our solvers)
-        if curr.h > lp_solver_factory.MAX_ALLOWED_HEURISTICS:
+        if curr.h > lp_solver.MAX_ALLOWED_HEURISTICS:
             continue
 
         # 12/10/2019: do it again, since the marking could be changed
@@ -448,7 +463,7 @@ def __search(sync_net, ini, fin, cost_function, skip, ret_tuple_as_trans_desc=Fa
         if curr.h < 0.01:
             if current_marking == fin:
                 return utils.__reconstruct_alignment(curr, visited, queued, traversed,
-                                               ret_tuple_as_trans_desc=ret_tuple_as_trans_desc)
+                                                     ret_tuple_as_trans_desc=ret_tuple_as_trans_desc)
 
         closed.add(current_marking)
         visited += 1
