@@ -1,9 +1,12 @@
 import pandas as pd
 
 from pm4py.objects.log import log as log_instance
-from pm4py.objects.conversion.log.versions import to_event_stream
+from pm4py.objects.conversion.log.variants import to_event_stream
+import deprecation
 
 
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='use objects.conversion.log.variants.to_data_frame')
 def apply(log, parameters=None):
     """
     Return a Pandas dataframe from a given log
@@ -26,6 +29,8 @@ def apply(log, parameters=None):
     return get_dataframe_from_event_stream(log)
 
 
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='use objects.conversion.log.variants.to_data_frame')
 def get_dataframe_from_event_stream(log):
     """
     Return a Pandas dataframe from a given log
@@ -43,7 +48,7 @@ def get_dataframe_from_event_stream(log):
     if isinstance(log, pd.core.frame.DataFrame):
         return log
     if type(log) is log_instance.EventLog:
-        log = to_event_stream.transform_event_log_to_event_stream(log)
+        log = to_event_stream.__transform_event_log_to_event_stream(log)
     transf_log = [dict(x) for x in log]
     df = pd.DataFrame.from_dict(transf_log)
 
