@@ -1,4 +1,5 @@
 import pandas
+import deprecation
 
 from pm4py.objects.conversion.log.versions import to_dataframe
 from pm4py.objects.log.exporter.parquet.versions import pandas as pandas_exporter
@@ -19,7 +20,8 @@ except:
     # Fastparquet is not installed
     pass
 
-
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='Use exporter module instead.')
 def apply(log, path, parameters=None, variant=DEFAULT_VARIANT):
     """
     Exports a log to a Parquet file
@@ -40,7 +42,8 @@ def apply(log, path, parameters=None, variant=DEFAULT_VARIANT):
 
     return VERSIONS[variant](log, path, parameters=parameters)
 
-
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='Use exporter module instead.')
 def export_log(log, path, parameters=None, variant=DEFAULT_VARIANT):
     """
     Exports a log to a Parquet file
@@ -61,7 +64,8 @@ def export_log(log, path, parameters=None, variant=DEFAULT_VARIANT):
 
     return VERSIONS[variant](log, path, parameters=parameters)
 
-
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='Use exporter module instead.')
 def export_df(log, path, parameters=None, variant=DEFAULT_VARIANT):
     """
     Exports a log to a Parquet file
