@@ -1,15 +1,18 @@
-from pm4py.util.dt_parsing import factory as dt_parse_factory
 import os
 
+import deprecation
+
 from pm4py.objects.log.log import EventLog, Trace, Event
 from pm4py.objects.log.util import sorting
+from pm4py.util.dt_parsing import factory as dt_parse_factory
 
 
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='this file/method is deprecated and moved to the variants package; use the importer entrypoint instead')
 def import_log(filename, parameters=None):
     """
     Import a log object from a XML file
     containing the traces, the events and the simple attributes of them
-
     Parameters
     -----------
     filename
@@ -22,7 +25,6 @@ def import_log(filename, parameters=None):
             index_trace_indexes -> Specify if trace indexes should be added as event attribute for each event
             max_no_traces_to_import -> Specify the maximum number of traces to import from the log
             (read in order in the XML file)
-
     Returns
     -----------
     xes
