@@ -1,6 +1,8 @@
 import shutil
+import deprecation
 
-
+@deprecation.deprecated(deprecated_in='1.3.0', removed_in='2.0.0', current_version='',
+                        details='Use visualizer module instead.')
 def save(gviz, output_file_path):
     """
     Save the diagram
@@ -12,5 +14,5 @@ def save(gviz, output_file_path):
     output_file_path
         Path where the GraphViz output should be saved
     """
-    render = gviz.render()
+    render = gviz.render(cleanup=True)
     shutil.copyfile(render, output_file_path)
